.nautilus-circular-button {
  border-radius: 20px;
  -gtk-outline-radius: 20px;
}

nautilus-desktop-window,
nautilus-desktop-window notebook,
nautilus-desktop-window notebook > stack {
    background: transparent;
}
